#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	LCDlib_SPI.c						//
//   	SPI functions For LCD			//
//////////////////////////////////////////


// Global Volatile Bits
volatile bit lcd_cs@LATC.0;			// Chip Select (low true)


/////////////////////////////////////
//	Function: SPILCD_init
//	Purpose:  initializes the SPI LCD
//	Input:    Void
//  Output:   Void
/////////////////////////////////////
void SPILCD_init(void){
	
	// Reset
	sspcon1 = 0b00000000;
	
	// SSP1STAT
		//bit7:   0: sample time, middle of period
		//bit6:   0: CKE clock select (rising)
		//bit5-0: 0: I2C mode only, Default is 0 
	sspstat = 0b00000000;
	
	// SSP1CON1
		//bit7:   0:    write collision, clear in software
		//bit6:   0:    receiver overflow, clear in software, avoid by always reading buffer
		//bit5:   1:    mssp1 enable bit
		//bit4:   1:    CKP - idle state is 0 or 1
		//bit3-0: 0010: Clock speed - Fos/64 
		//		  0011: TMR2/2
	sspcon1 = 0b00110010; 	
		// Sets data in, data out, and clock ports
		// BUT: must clear trisc.5 For data out
		//	    must clear trisc.3 For clock 
	
	// I/O Ports
		/* SCK:   C3 --> Output	0	
	   	   MISO:  C4 --> Input	1	
	   	   MOSI:  C5 --> Output	0	
		   Select:C0 --> Output 0   */
	trisc.5 = 0;
	trisc.3 = 0;
	trisc.0 = 0;
	
	// Set Default Output Values
	lcd_cs = 1;		// Set chip select high
	}
	


////////////////////////////////////////////////
//	Function: SPILCD_putc
//	Purpose:  Puts a char to the SPI LCD
//	Input:    Void
//  Output:   c -> the string to put to the screen
////////////////////////////////////////////////
void SPILCD_putc(char c){
	char ans;
	delay_ms(1);
	lcd_cs = 0;
	delay_ms(1);
	ans = spi_byte(c);
	delay_ms(1);
	lcd_cs = 1;
	delay_ms(1);
	return;
	}


	
//////////////////////////////////////////////////////////////////
//	Function: SPILCD_putShort
//	Purpose:  prints an unsigned short to the lcd in decimal form
//	Input:    dec -> that value to be printed
//	Output:   Void
//////////////////////////////////////////////////////////////////
void SPILCD_putShort(unsigned short dec) {
	unsigned short dec1;		// Ten Thousands place
	unsigned short dec2;		// Thousands place
	unsigned short dec3;		// Hundreds place
	unsigned short dec4;		// Tens place
	unsigned short dec5;		// Ones place
	
	dec1 = dec/10000;			// Get the Ten Thousands place with division
	dec = dec%10000;			// Calculate how much is left
	dec2 = dec/1000;			// Get the Thousands place with division
	dec = dec%1000;				// Calculate how much is left
	dec3 = dec/100;				// Get the Hundreds place with division
	dec = dec%100;				// Calculate how much is left
	dec4 = dec/10;				// Get the Tens place with division
	dec5 = dec%10;				// The Ones place is how much is left
	
	if(dec1 != 0)				// If there is a Ten Thousands place, print it
		SPILCD_putc(dec1 + '0');
	if(dec2 != 0)				// If there is a Thousands place, print it
		SPILCD_putc(dec2 + '0');
	if(dec3 != 0)				// If there is a Hundreds place, print it
		SPILCD_putc(dec3 + '0');
	if(dec4 != 0)				// If there is a Tens place, print it
		SPILCD_putc(dec4 + '0');
	SPILCD_putc(dec5 + '0');	// Print the one's place
	}



////////////////////////////////////////////////
//	Function: SPILCD_printf
//	Purpose:  Puts a string to the SPI LCD
//  Input:    c -> the char to put to the screen
//  Output:   Void
////////////////////////////////////////////////
void SPILCD_printf(char c[]){
	int i;
	for(i=0; c[i] != '\0'; i++)		// Loop through each character in the array
		SPILCD_putc(c[i]);			// Put the character to the terminal
		delay_ms(1);
	return;
	}



////////////////////////./////////////////////////
// Function: SPILCD_cmd
// Purpose:  Sends a command to the screen via SPI
// Input:    The command length and the command
// Output:   Void
//////////////////////////////////////////////////
void SPILCD_cmd(char length, char* cmd) {
	char ans;
	int i;
	SPILCD_putc(0xFE);
	for (i=0;i<length;i++) {
		SPILCD_putc(cmd[i]);
		delay_ms(1);
		}
	}		



/////////////////////////////////////////////////
// Function: SPILCD_clear
// Purpose:  Clears the screen and returns cursor
// Input:    Void
// Output:   Void
/////////////////////////////////////////////////
void SPILCD_clear(void) {
	char cmd[1];
	cmd[0] = 0x51;
	SPILCD_cmd(1,cmd);
	}



///////////////////////////////////////////
// Function: SPILCD_newline
// Purpose:  Moves the LCD cursor to line 2
// Input:    Void
// Output:   Void
///////////////////////////////////////////
void SPILCD_newline(void) {
	char cmd[2];
	cmd[0] = 0x45;
	cmd[1] = 0x40;
	SPILCD_cmd(2,cmd);
	}


///////////////////////////////////////////
// Function: SPILCD_home
// Purpose:  Moves the LCD cursor to line 1
// Input:    Void
// Output:   Void
///////////////////////////////////////////
void SPILCD_home(void) {
	char cmd[1];
	cmd[0] = 0x46;
	SPILCD_cmd(1,cmd);
	}


//////////////////////////////////////////////
// Function: SPILCD_brightness
// Purpose:  Sets the LCD backlight brightness
// Input:    The brightness setting 1-8
// Output:   Void
//////////////////////////////////////////////
void SPILCD_brightness(char brightness) {
	char cmd[2];
	cmd[0] = 0x53;
	cmd[1] = brightness;
	SPILCD_cmd(2,cmd);
	}
