#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	RTClib.c							//
//  Functions for the RTC				//
//////////////////////////////////////////


// Global Volatile Bits
volatile bit rtc_en@LATD.7;			// Chip Enable

// Function Prototypes in RTClib.h


//////////////////////////////////////
//	Function: RTC_init
//	Purpose:  initializes the SPI RTC
//  Inputs:   Void
//  Output:   Void
/////////////////////////////////////
void RTC_init(void){
	
	// Reset
	sspcon1 = 0b00000000;
	
	// SSPSTAT
		//bit7:   0: sample time, middle of period
		//bit6:   0: CKE clock select (Falling if CKP is 0)
		//bit5-0: 0: I2C mode only, Default is 0 
	sspstat = 0b00000000;
	
	// SSPCON1
		//bit7:   0:    write collision, clear in software
		//bit6:   0:    receiver overflow, clear in software, avoid by always reading buffer
		//bit5:   1:    mssp1 enable bit
		//bit4:   0:    CKP - idle state is 0 or 1
		//bit3-0: 0010: Clock speed - Fos/64
	sspcon1 = 0b00100010; 	
		// Sets data in, data out, and clock ports
		// BUT: must clear trisc.5 For data out
		//	must clear trisc.3 For clock 
	
	// I/O Ports
		/* Enable:D7 --> Output	0
	   	   SCK:   C3 --> Output	0	
	   	   MISO:  C4 --> Input	1	
	   	   MOSI:  C5 --> Output	0	*/
	trisc &= 11010111b;
	trisd.7 = 0;
		
	// Set Default Output Values
	rtc_en = 0;		// Set chip select high
	
	// Set up the initial parameters
		// Control Register
		// Bit 7: 	Enable Oscillator (Low True)	0 
		// Bit 6: 	Write Protect					1
		// Bit 5-3:	Reserved						000
		// Bit 2:	Int Con (Use both Int Pins?)	1
		// Bit 1:	Alarm Enable 1					0
		// Bit 0:	Alarm Enable 0					0
	char ans;
	ans = RTC_reg(0x8F,0b01000100);

	}
	
	
//////////////////////////////////////////////
//	Function: RTC_reg
//	Purpose:  Writes or reads an RTC register
//	Input:    Address to access, Byte to write
//  Output:   Byte read
//////////////////////////////////////////////
char RTC_reg(char add, char input){
	char output = 0;
	rtc_en = 1;					// Set the chip enable low
	output = spi_byte(add);	// Send an spi byte with address
	output = spi_byte(input);	// Send an spi byte with the data
	rtc_en = 0;					// Set the chip enable low
	return output;
	}


/////////////////////////////////////////////////////////////////
//  Function: RTC_set
//  Purpose:  Sets the current time to the RTC
//  Inputs:   Pointer to a time structure
//  Outputs:  Void
/////////////////////////////////////////////////////////////////
void RTC_set(struct time* tp) {
	char i;			// Loop variable
	char ans;		// Temporary register placeholder
	char time[7];	// The time information to be sent by SPI
	
	time[0] = ((tp->sec10 <<4) & 0b01110000) | (tp->sec1  & 0b00001111);	// See datasheet
	time[1] = ((tp->min10 <<4) & 0b01110000) | (tp->min1  & 0b00001111);
	time[2] = ((tp->hr10  <<4) & 0b00110000) | (tp->hr1   & 0b00001111);
	time[3] = tp->day & 0b00001111;
	time[4] = ((tp->date10<<4) & 0b00110000) | (tp->date1 & 0b00001111);
	time[5] = ((tp->mon10 <<4) & 0b00110000) | (tp->mon1  & 0b00001111);
	time[6] = ((tp->yr10  <<4) & 0b11110000) | (tp->yr1   & 0b00001111);
	
	ans = RTC_reg(0x8F,0b00000100);	// Disable write protect
	
	rtc_en = 1;						// Send time bytes over SPI (Burst Mode)
	ans = spi_byte(0x80);			// First address to write to
	for (i=0;i<7;i++) {				
		ans = spi_byte(time[i]);	// Write the 7 bytes (address inc. automatically)
		}
	rtc_en = 0;						// End burst mode
	
	ans = RTC_reg(0x8F,0b01000100);	// Enable write protect
	return;
	}



////////////////////////////////////////////////
//  Function: RTC_get
//  Purpose:  Gets the current time from the RTC
//  Input:    Pointer to a time structure
//  Output:   Void
/////////////////////////////////////////////////
void RTC_get(struct time* tp) {
	char tmp;		// Temporary storage variable
	struct time t;	// Structure to hold the output
	
	tmp = RTC_reg(0x00,0);					// Read the seconds register
	tp->sec10 = (tmp>>4) & 0b00000111;		// Decode the 10s place
	tp->sec1  = tmp & 0b00001111;			// Decode the 1s place
	tp->sec   = 10*tp->sec10 + tp->sec1;	// The total seconds
	
	tmp = RTC_reg(0x01,0);					// Read the minutes register
	tp->min10 = (tmp>>4) & 0b00000111;		// Decode the 10s place
	tp->min1  = tmp & 0b00001111;			// Decode the 1s place
	tp->min   = 10*tp->min10 + tp->min1;	// The total minutes
					
	tmp = RTC_reg(0x02,0);					// Read the hours register
	tp->hr10  = (tmp>>4) & 0b00000011;		// Decode the 10s place
	tp->hr1   = tmp & 0b00001111;			// Decode the 1s place
	tp->hr    = 10*tp->hr10 + tp->hr1;		// The total hour
					
	tmp = RTC_reg(0x03,0);					// Read the days register
	tp->day   = tmp & 0b00001111;			// Decode the 1s place
			
	tmp = RTC_reg(0x04,0);					// Read the date register
	tp->date10 = (tmp>>4) & 0b00000011;		// Decode the 10s place
	tp->date1  = tmp & 0b00001111;			// Decode the 1s place
	tp->date   = 10*tp->date10 + tp->date1;	// The total date
					
	tmp = RTC_reg(0x05,0);					// Read the month register
	tp->mon10  = (tmp>>4) & 0b00000011;		// Decode the 10s place
	tp->mon1   = tmp & 0b00001111;			// Decode the 1s place						
	tp->mon    = 10*tp->mon10 + tp->mon1;	// The total month
					
	tmp = RTC_reg(0x06,0);					// Read the year register
	tp->yr10  = (tmp>>4) & 0b00001111;		// Decode the 10s place
	tp->yr1   = tmp & 0b00001111;			// Decode the 1s place
	tp->yr    = 10*tp->yr10 + tp->yr1;		// The total year
	return;
	}



////////////////////////////////////////////////////////
//  Function: RTC_print_terminal
//  Purpose:  Prints a time to the screen through usart2
//  Input:    A Pointer to a time structure
//  Output:   Void
////////////////////////////////////////////////////////
void RTC_print_terminal(struct time* tp) {
	usart_printf("\n\r");
	usart_putShort(tp->hr10);	// Put the hours to the screen
	usart_putShort(tp->hr1 );	// Put the hours to the screen
	usart_printf(":");	
	usart_putShort(tp->min10);	// Put the minutes to the screen
	usart_putShort(tp->min1 );	// Put the minutes to the screen
	usart_putc(':');
	usart_putShort(tp->sec10);	// Put the seconds to the screen
	usart_putShort(tp->sec1 );	// Put the seconds to the screen
	usart_printf(" ");
				
	switch (tp->day) {		// Put the day of the week to the screen
		case 1:
			usart_printf("Sunday ");
			break;
		case 2:
			usart_printf("Monday ");
			break;
		case 3:
			usart_printf("Tuesday ");
			break;
		case 4:
			usart_printf("Wednesday ");
			break;
		case 5:
			usart_printf("Thursday ");
			break;
		case 6:
			usart_printf("Friday ");
			break;
		case 7:
			usart_printf("Saturday ");
			break;	
		default:
			usart_printf("Error ");
			break;											
			}
					
	switch (tp->mon) {		// Put the month to the sreen
		case 1:
			usart_printf("January ");
			break;
		case 2:
			usart_printf("February ");
			break;
		case 3:
			usart_printf("March ");
			break;
		case 4:
			usart_printf("April ");
			break;
		case 5:
			usart_printf("May ");
			break;
		case 6:
			usart_printf("June ");
			break;
		case 7:
			usart_printf("July ");
			break;
		case 8:
			usart_printf("August ");
			break;
		case 9:
			usart_printf("September ");
			break;
		case 10:
			usart_printf("October ");
			break;
		case 11:
			usart_printf("November ");
			break;
		case 12:
			usart_printf("December ");
			break;							
		default:
			usart_printf("Error ");
			break;
		}
		
	usart_putShort(tp->date10);// Put the date to the screen
	usart_putShort(tp->date1 );// Put the date to the screen
	usart_printf(", 20");
	usart_putShort(tp->yr10);	// Put the year to the screen
	usart_putShort(tp->yr1 );	// Put the year to the screen
	}


//////////////////////////////////////////////////
//  Function: RTC_print_lcd
//  Purpose:  Prints a time to the lcd through spi
//  Input:    A Pointer to a time structure
//  Output:   Void
//////////////////////////////////////////////////
void RTC_print_lcd(struct time* tp) {
	SPILCD_clear();
	SPILCD_putShort(tp->hr10);	// Put the hours to the screen
	SPILCD_putShort(tp->hr1 );	// Put the hours to the screen
	SPILCD_printf(":");	
	SPILCD_putShort(tp->min10);	// Put the minutes to the screen
	SPILCD_putShort(tp->min1 );	// Put the minutes to the screen
	SPILCD_putc(':');
	SPILCD_putShort(tp->sec10);	// Put the seconds to the screen
	SPILCD_putShort(tp->sec1 );	// Put the seconds to the screen
	SPILCD_newline();
				
	switch (tp->day) {		// Put the day of the week to the screen
		case 1:
			SPILCD_printf("Sun ");
			break;
		case 2:
			SPILCD_printf("Mon ");
			break;
		case 3:
			SPILCD_printf("Tues ");
			break;
		case 4:
			SPILCD_printf("Wed ");
			break;
		case 5:
			SPILCD_printf("Thur ");
			break;
		case 6:
			SPILCD_printf("Fri ");
			break;
		case 7:
			SPILCD_printf("Sat ");
			break;	
		default:
			SPILCD_printf("Err ");
			break;											
			}
					
	switch (tp->mon) {		// Put the month to the sreen
		case 1:
			SPILCD_printf("Jan");
			break;
		case 2:
			SPILCD_printf("Feb ");
			break;
		case 3:
			SPILCD_printf("Mar ");
			break;
		case 4:
			SPILCD_printf("Apr ");
			break;
		case 5:
			SPILCD_printf("May ");
			break;
		case 6:
			SPILCD_printf("June ");
			break;
		case 7:
			SPILCD_printf("July ");
			break;
		case 8:
			SPILCD_printf("Aug ");
			break;
		case 9:
			SPILCD_printf("Sept ");
			break;
		case 10:
			SPILCD_printf("Oct ");
			break;
		case 11:
			SPILCD_printf("Nov ");
			break;
		case 12:
			SPILCD_printf("Dec ");
			break;							
		default:
			SPILCD_printf("Err ");
			break;
		}
		
	SPILCD_putShort(tp->date10);// Put the date to the screen
	SPILCD_putShort(tp->date1 );// Put the date to the screen
	SPILCD_printf(", 20");
	SPILCD_putShort(tp->yr10);	// Put the year to the screen
	SPILCD_putShort(tp->yr1 );	// Put the year to the screen
	}