#include <system.h>
#include "ZigBeelib.h"
#include "LCDlib_SPI.h"
#include "RTClib.h"
#include "ADlib.h"
#include "usartlib.h"
#include "EEPROMlib.h"
#include "motorlib.h"

//////////////////////////////////////////
//	Smart Windows Project				//
//										//
//	ZigBeelib.c							//
//   	SPI functions for ZigBee		//
//////////////////////////////////////////

// Function Prototypes are in ZigBeelib.h

// Global Volatile Bits
volatile bit zb_cs @PORTD.0;	// Chip Select (low true)
volatile bit zb_rs @PORTD.1;	// Reset (Low true)
volatile bit zb_slp@PORTD.2;	// Sleep Signal


///////////////////////////////////////
//	Function: spi_init
//	Purpose:  Initializes MSSP for SPI
//	Inputs:   rate -> currently unused
//	Output:   Void				 
///////////////////////////////////////
void spi_init(unsigned short rate)
{
	// Reset
	sspcon1 = 0b00000000;
	
	// SSP1STAT
		//bit7:   0: sample time, middle of period
		//bit6:   1: CKE clock select, (Words-0, Picture-1)
		//bit5-0: 0: I2C mode only, Default is 0 
	sspstat = 0b01000000;
	
	// SSP1CON1
		//bit7:   0:    write collision, clear in software
		//bit6:   0:    receiver overflow, clear in software, avoid by always reading buffer
		//bit5:   1:    mssp enable bit
		//bit4:   0:    CKP - idle state is 0 or 1
		//bit3-0: 0010: Clock speed - Fos/64
	sspcon1 = 0b00100010; 	
		// Sets data in, data out, and clock ports
		// BUT: must clear trisc.5 For data out
		//	    must clear trisc.3 For clock 
	
	// I/O Ports
		/* SEL:   D0 --> Output 0	
		   Reset: D1 --> Output 0	
		   SLP:   D2 --> Output 0		   
		   IRQ:   D3 --> Input	1
	   	   SCK:   C3 --> Output	0	
	   	   MISO:  C4 --> Input	1	
	   	   MOSI:  C5 --> Output	0	*/
	trisc.5 = 0;
	trisc.3 = 0;
	trisd.0 = 0;
	trisd.1 = 0;
	trisd.2 = 0;
	trisd.3 = 1;
	
	// Set Default Output Values
	zb_cs  = 1;		// Set chip select high
	zb_slp = 0;		// Set slp signal low
	zb_rs  = 1;		// Set reset high
}



//////////////////////////////////////////////////////////////
//	Function: spi_byte
//	Purpose:  Sends a byte on MOSI and recieves a byte on MISO
//	Inputs:   data -> the byte to be sent on MOSI
//	Ouput:	  The byte received on MISO
//////////////////////////////////////////////////////////////
char spi_byte(char data)
{
	volatile bit sspif@PIR1.3;	// Define a bit For interuppt flag	
	char return_byte;
	// Collision Check not required yet
	sspif = 0;					// Set interrupt flag to low
	sspbuf = data;				// Write data to buffer to start trasmission
	while(!sspif);				// Wait until interupt flag goes high
	return_byte = sspbuf;
	return return_byte;			// Return the new contents of the buffer
}



////////////////////////////////////////////////////////////
//	Function: 	zb_init
//	Purpose:  	Initializes the zigbee card for transmission
//	Inputs:   	channel
//	Output:  	Void				 
////////////////////////////////////////////////////////////
void zb_init(char channel)
{
	char reg;
	
	// Reset the device
	zb_rs = 0;
	delay_ms(20);
	zb_rs = 1;
	delay_ms(20);
	
	zb_write_reg(IRQ_MASK,0b00000000);			// Mask all interrupts
	reg = zb_read_reg(IRQ_STATUS);				// Read the interrupt register to clear it
	zb_write_reg(PHY_CC_CCA,channel);			// Set the communication channel
	zb_state_trans(TRX_OFF,TRX_OFF);			// Move ZigBee card to clock state with command TRX_OFF
	zb_state_trans(PLL_ON,PLL_ON);				// Move to PLL-ON state
	
	// Initialize for Extended Mode
	unsigned short i;					// Loop Variable
	char read_reg;						// Temporary Register
	char write_reg =0;					// Temporaty Register
	for (i=0;i<4;i++) {					// Produce an 8-bit random number from register 6
		read_reg = zb_read_reg(PHY_RSSI);
		switch (i){
			case 0:
				write_reg.0 = read_reg.6;
				write_reg.1 = read_reg.5;
			break;
			case 1:
				write_reg.2 = read_reg.6;
				write_reg.3 = read_reg.5;			
			break;
			case 2:
				write_reg.4 = read_reg.6;
				write_reg.5 = read_reg.5;			
			break;
			case 3:
				write_reg.6 = read_reg.6;
				write_reg.7 = read_reg.5;			
			break;
			}
		nop();
		nop();
		nop();
		}
	zb_write_reg(0x2D,write_reg);		// Write the random number to 0x2D
	write_reg = 01000111b;				// Write the random number to 0x2E <2:0>, keep bit 6 set
	zb_write_reg(0x2E,write_reg);
}



////////////////////////////////////////////////////////////
//	Function: zb_read_reg
//	Purpose:  Reads a register on the ZigBee card
//	Inputs:   add -> the register address to read from
//	Output:   The byte sitting in the register that was read
////////////////////////////////////////////////////////////
char zb_read_reg(char add)
{
	char ans;							// Temporary 1-byte variable
	zb_cs = 0;   						// Chip select low to begin process
	ans = spi_byte((add&0x3F)|0x80);	// Send command byte 10__add_
	ans = spi_byte(0);					// Send dummy byte to collect contents
	zb_cs = 1;   						// Chip select high to end transmission
	return ans;
}



/////////////////////////////////////////////////////////
//	Fucntion: zb_write_reg
//	Purpose:  Writes to a register on the ZigBee card
//	Inputs:   add  -> Address of the register to write to
//	          data -> Byte to put in the register
//	Output:	  Void
/////////////////////////////////////////////////////////
void zb_write_reg(char add, char data)
{
	char ans;							// Temporary 1-byte variable
	zb_cs = 0;   						// Chip select low to begin transmission	
	ans = spi_byte((add&0x3F)|0xC0); 	// Send command byte 11___add_
	ans = spi_byte (data);				// Send data to be written
	zb_cs = 1;   						// Chip select high to end transmission
}



/////////////////////////////////////////////////////////////////////
//	Function: 	zb_write_fb
//	Purpose:  	Writes a 1-byte payload to the ZigBee card frame buffer
//	          	with the appropriate 802.15.4-2003 format
//	Input:    	data -> The data bytes to be included in the buffer
//				size -> The size in bytes of the data
//				seq -> The current seq value
//				add0 -> First byte of the address to send to
//				add1 -> Second byte of the address to send to
//	Output:   	The updated seq value	
///////////////////////////////////////////////////////////////////// 
char zb_write_fb(char add1,char add0,char data[],char size,char seq)
{
	char ans;				// Temporary 1-byte variable
	int i;					// Loop variable
	
	char my_pan_0 = zb_read_reg(PAN_ID_0);
	char my_pan_1 = zb_read_reg(PAN_ID_1);	
	char my_add_0 = zb_read_reg(SHORT_ADDR_0);
	char my_add_1 = zb_read_reg(SHORT_ADDR_1);
	
	zb_cs = 0;   			// Chip select low to begin transmission  
	ans = spi_byte(0x60);	// Send command byte 0110 0000
	ans = spi_byte(13+size);// Send PHR (frame length) -> 13 + size of data
		
	// Send PSDU
	ans = spi_byte(0b00100001);	// Send FCF(control bytes) 7654 3210 FEDC BA98
	ans = spi_byte(0x88);			// 0010 0001 1000 1000
	
	ans = spi_byte(seq); 		// Send sequence field
	
	ans = spi_byte(my_pan_0);	// Destination Send addressing fields
	ans = spi_byte(my_pan_1);	
	ans = spi_byte(add0);
	ans = spi_byte(add1);
	
	ans = spi_byte(my_pan_0);	// Source Send addressing fields
	ans = spi_byte(my_pan_1);	
	ans = spi_byte(my_add_0);
	ans = spi_byte(my_add_1);
	
	/* */						// No Security Fields
	
	for (i=0;i<size;i++)		// Send data bytes
	{
		ans = spi_byte(data[i]);	
	}
	
	/* */						// Auto sends the 2-byte FCS

	zb_cs = 1; 					// Chip select high to end transmission
	
	seq++;						// Increment seq
	return seq;
}



////////////////////////////////////////////////////////////////////
//	Function: zb_read_fb
//	Purpose:  Reads the ZigBee frame buffer assuming the format used
//		  	  used in the function zb_write_fb (see above)
//	Input:    Pointer to the memory where frame buffer will be stored
//	Output:   The length of the framebuffer read out
////////////////////////////////////////////////////////////////////
char zb_read_fb(char* tmp)
{
	char length;				// Holds the contents of PHR field
	zb_cs = 0;   				// Chip select low to begin transmission
	length = spi_byte(0x20);	// Send command byte 0010 0000
	
	length = spi_byte(0);		// Read and store the PHR
	tmp[0] = length;
	
	char ans;					// Temporary variable
	char i;						// Loop counter variable
	for (i=0; i<length; i++)
	{
		ans = spi_byte(0);		// Read the next byte
		tmp[i+1] = ans;			// Store that byte
	}
	
	zb_cs = 1; 					// Chip select high to end transmission
	return length+1;
}



//////////////////////////////////////////////////////////////
//	Function: 	zb_state_trans
//	Purpose:  	Moves the zb card into a new state
//	Inputs:   	state -> the state you want to move into
//				command -> the command to give to the register
//	Output:  	Void				 
//////////////////////////////////////////////////////////////
void zb_state_trans(char state, char command)
{
	char reg;
	zb_write_reg(TRX_STATE,command);		// Move ZigBee card to PLL-LOCK
 	reg = zb_read_reg(TRX_STATUS);
 	while((reg&0b00011111) != state)
 	{
		reg = zb_read_reg(TRX_STATUS);
 	}
}



/////////////////////////////////////////////////////////////
//	Function: 	zb_tx
//	Purpose:  	Transmits a char string over ZigBee
//	Inputs:   	data -> the data to send over ZigBee
//				size -> the size of the data to send
//				seq -> the current seq value
//				add0 -> First byte of the address to send to
//				add1 -> Second byte of the address to send to
//	Output:  	the incremented seq value      
/////////////////////////////////////////////////////////////
char zb_tx (char add1,char add0,char data[],char size,char seq)
{
	char reg;
	char success = 0;
	zb_state_trans(TX_ARET_ON,TX_ARET_ON);		// Move to Extended TX state
	seq = zb_write_fb(add1,add0,data,size,seq);	// Write the data into framebuffer
	zb_state_trans(BUSY_TX_ARET,TX_START);		// Begin the transmission
	reg = zb_read_reg(TRX_STATUS);				// Wait until transmit is finished by returning from Busy
	while(reg != TX_ARET_ON){
		reg = zb_read_reg(TRX_STATUS);
		}
	reg = zb_read_reg(TRX_STATE);			// Extended-mode transmission result
	if ((reg&11100000b)==0) {
		// Reg 7:5 will be 0 on success
		success = 1;
		}
	
	zb_state_trans(PLL_ON,PLL_ON);			// Move back to the TRX_OFF state
	return success;
}



////////////////////////////////////////////////////
//	Function: 	zb_rx
//	Purpose:  	Receives a byte over ZigBee
//	Inputs:   	Length to listen for (0 for infinite)
//	Output:  	Whether a frame was received				              
////////////////////////////////////////////////////
char zb_rx (unsigned short length)
{
	char reg;
	char success = 0;

	zb_write_reg(IRQ_MASK,0b00101100); 			// Enable interrupts 2, 3, and 5
	reg = zb_read_reg(IRQ_STATUS);				// Refresh the intrupt buffer
	
	zb_state_trans(RX_AACK_ON,RX_AACK_ON);		// Move to the AACK Receive state
	
	// On message detect, auto move to BUSY_RX_AACK state
		// IRQ_2: When RX begins
		// IRQ_3: When TRX ends
		bool IRQ3= 0;
		// Puts FCS in PHY_RSSI bit 7
		// IRQ_5: If address match
		// Moves back into RX_AACK_ON
	
	switch(length) {
		case 0:									// Unending listen
			while (!IRQ3) {
				reg = zb_read_reg(IRQ_STATUS);
				if (reg.3==1){
					IRQ3=1;
					}
				}
			reg = zb_read_reg(TRX_STATUS);		// Wait until transmit is finished by returning from Busy
			while(reg != RX_AACK_ON){
					reg = zb_read_reg(TRX_STATUS);
					}
			reg = zb_read_reg(TRX_STATE);		// Extended-mode transmission result
			if (((reg&11100000)==0) | ((reg&11100000)==32)) {
				// Reg 7:5 will be 0 or 1 on success
				success = 1;
				}
		break;
		
		default:								// Limited listen
			unsigned short i;
			for (i=0;i<length;i++) {
				reg = zb_read_reg(IRQ_STATUS);
				if (reg.3==1){
					reg = zb_read_reg(TRX_STATUS);		// Wait until transmit is finished by returning from Busy
					while(reg != RX_AACK_ON){
						reg = zb_read_reg(TRX_STATUS);
						}
					reg = zb_read_reg(TRX_STATE);		// Extended-mode transmission result
					if (((reg&11100000)==0) | ((reg&11100000)==32)) {
						// Reg 7:5 will be 0 or 1 on success
						success = 1;
						}
					break;
					}
				}
		break;
		}
	
	zb_state_trans(PLL_ON,PLL_ON);
	zb_write_reg(IRQ_MASK,0x00);
	return success;
}



/////////////////////////////////////////////
//	Function: 	zb_sleep
//	Purpose:  	Puts the Zigbee card to sleep
//	Inputs:   	Void
//	Output:  	Void				              
/////////////////////////////////////////////
void zb_sleep(void)
{
	
	char reg;                            // Read current state
 	reg = zb_read_reg(TRX_STATUS);
 	if((reg&0b00011111) != TRX_OFF)		 // If not in TRX_OFF...
 	{
		zb_state_trans(TRX_OFF, TRX_OFF); // ...Move to TRX_OFF
		reg = zb_read_reg(TRX_STATUS);
	}
	zb_slp = 1; // Once in TRX_OFF, go to sleep with sleep signal port
	delay_ms(1);
}


////////////////////////////////////////
//	Function: 	zb_wake
//	Purpose:  	Wakes up the Zigbee card
//	Inputs:   	Void
//	Output:  	Void				              
////////////////////////////////////////
void zb_wake(void)
{
	zb_slp = 0;
	delay_ms(1);
}


/////////////////////////////////////////////////////////////////////
//	Function: 	zb_broadcast_fb
//	Purpose:  	Writes a broadcast payload to the ZigBee frame buffer
//	          	with the appropriate 802.15.4-2003 format
//	Input:    	data -> The data bytes to be included in the buffer
//				size -> The size in bytes of the data
//				seq -> The current seq value
//	Output:   	The updated seq value	
///////////////////////////////////////////////////////////////////// 
char zb_broadcast_fb(char data[],char size,char seq)
{
	char ans;				// Temporary 1-byte variable
	int i;					// Loop variable
	
	char my_pan_0 = zb_read_reg(PAN_ID_0);
	char my_pan_1 = zb_read_reg(PAN_ID_1);	
	char my_add_0 = zb_read_reg(SHORT_ADDR_0);
	char my_add_1 = zb_read_reg(SHORT_ADDR_1);
	
	zb_cs = 0;   			// Chip select low to begin transmission  
	ans = spi_byte(0x60);	// Send command byte 0110 0000
	ans = spi_byte(13+size);// Send PHR (frame length) -> 13 + size of data 
		
	// Send PSDU
	ans = spi_byte(0x01);		// Send FCF(control bytes) 7654 3210 FEDC BA98
	ans = spi_byte(0x88);			// 0000 0001 1000 1000
	
	ans = spi_byte(seq); 		// Send sequence field
	
	ans = spi_byte(0xFF);		// Destination Send addressing fields
	ans = spi_byte(0xFF);	
	ans = spi_byte(0xFF);
	ans = spi_byte(0xFF);
	
	ans = spi_byte(my_pan_0);	// Source Send addressing fields
	ans = spi_byte(my_pan_1);	
	ans = spi_byte(my_add_0);
	ans = spi_byte(my_add_1);
	
	/* */						// No Security Fields
	
	for (i=0;i<size;i++)		// Send data bytes
	{
		ans = spi_byte(data[i]);	
	}
	
	/* */						// Auto sends the 2-byte FCS

	zb_cs = 1; 					// Chip select high to end transmission
	
	seq++;						// Increment seq
	return seq;
}


/////////////////////////////////////////////////////////////
//	Function: 	zb_broadcast
//	Purpose:  	Transmits a char string over ZigBee
//	Inputs:   	data -> the data to send over ZigBee
//				size -> the size of the data to send
//				seq -> the current seq value
//	Output:  	the incremented seq value      
/////////////////////////////////////////////////////////////
char zb_broadcast (char data[],char size,char seq)
{
	char reg;
	
	zb_state_trans(TX_ARET_ON,TX_ARET_ON);		// Move to Extended TX state
	seq = zb_broadcast_fb(data,size,seq);	// Write the data into framebuffer
	
	zb_state_trans(BUSY_TX_ARET,TX_START);	// Begin the transmission
	reg = zb_read_reg(TRX_STATUS);			// Wait until transmit is finished by returning from Busy
	while(reg != TX_ARET_ON){
		reg = zb_read_reg(TRX_STATUS);
		}
	/*
	reg = zb_read_reg(TRX_STATE);			// Extended-mode transmission result
	if (((reg&11100000)==0) | ((reg&11100000)==32)) {
		// Reg 7:5 will be 0 or 1 on success
		success = 1;
		}
	*/			
	zb_state_trans(PLL_ON,PLL_ON);			// Move back to the TRX_OFF state
	return seq;
}